kdebig1_function(h, pts, poly = NA, kernel = "normal", grid = T, x = NA,
	y = NA, nx = NA, ny = NA, edge = T, scale = T, inpoly = NA, 
	rings = 10, npt = 10 * rings)
{

# Julia Kelsall

	rx <- pts[, 1]
	ry <- pts[, 2]
	nr <- length(rx) # no of data points
	pol <- !is.na(poly[1]) # is polygon specified?
	if(pol) {
		if(sum(inout(pts, poly))!=nr)
			stop("Observations outside polygon")
	}
	if(!pol) {
		edge <- F # can't do edge corrections if no polygon
		scale <- F # can't scale either
		if((is.na(x[1])) & (is.na(nx))) {
			grid <- T
			xrange_(max(rx) - min(rx) + 4 * h)
			xmin_min(rx)-2*h
			nx_max(20,ceiling(xrange/(h/2)))
			yrange_(max(ry) - min(ry) + 4 * h)
			ymin_min(ry)-2*h
			ny_max(20,ceiling(yrange/(h/2)))
			x <- ((1:nx) - 1/2)/nx * xrange + xmin
			y <- ((1:ny) - 1/2)/ny * yrange + ymin
		}
	}
	if((pol) & (is.na(x[1])) & (is.na(nx))) {
		edge <- T
		scale <- T
		xmin <- min(poly[, 1])
		xmax <- max(poly[, 1])
		ymin <- min(poly[, 2])
		ymax <- max(poly[, 2])
		xrange <- xmax - xmin
		yrange <- ymax - ymin
		nx_max(20,ceiling(xrange/(h/2)))
		ny_max(20,ceiling(yrange/(h/2)))
		x <- ((1:nx) - 1/2)/nx * xrange + xmin
		y <- ((1:ny) - 1/2)/ny * yrange + ymin
	}
	if(grid == T) {
		if(is.na(x[1])) {
			xmin <- min(poly[, 1])
			xmax <- max(poly[, 1])
			ymin <- min(poly[, 2])
			ymax <- max(poly[, 2])
			xrange <- xmax - xmin
			yrange <- ymax - ymin
			x <- ((1:nx) - 1/2)/nx * xrange + xmin
			y <- ((1:ny) - 1/2)/ny * yrange + ymin
		}
		if(is.na(nx)) {
			nx <- length(x)
			ny <- length(y)
		}
		ans <- list(0, 0, matrix(nrow = nx, ncol = ny), 0)
		names(ans) <- c("x", "y", "z", "scale")
		ans$x <- x
		ans$y <- y
		ans$scale <- 1
	
	if(pol&(h==Inf)){
		ans$z_matrix(1/area(poly),nrow=nx,ncol=ny)
		xyvec <- cbind(rep(x, ny), rep(y, rep(nx, ny)))
		lpmat1 <- matrix(inout(xyvec, poly), nrow = nx,
			ncol=ny,byrow=F)
		ans$z[!lpmat1]_NA
	}
	else{
		
		if(edge == T) {
			edgexy <- kdebig2(h, x, y, poly, kernel, grid, inpoly, 
				rings, npt)
		}
		zmat <- matrix(0, nrow = nx, ncol = ny)
		storage.mode(zmat) <- "single"
		if(kernel == "normal") {
			if(!is.loaded(symbol.For("kdesr2"))){dyn.load(
			"/home/morrisse/libjek/Binreg/Bazza/kdesr2.o")}
			result <- .Fortran("kdesr2",
				h = as.single(h),
				rx = as.single(rx),
				ry = as.single(ry),
				nr <- as.integer(nr),
				x = as.single(x),
				nx = as.integer(nx),
				y = as.single(y),
				ny = as.integer(ny),
				zmat = zmat)
		}
		if(kernel == "quartic") {
			if(!is.loaded(symbol.For("kdesr5"))){dyn.load(
			"/home/morrisse/libjek/Binreg/Bazza/kdesr5.o")}
			result <- .Fortran("kdesr5",
				h = as.single(h),
				rx = as.single(rx),
				ry = as.single(ry),
				nr <- as.integer(nr),
				x = as.single(x),
				nx = as.integer(nx),
				y = as.single(y),
				ny = as.integer(ny),
				zmat = zmat)
		}
		fhat <- result$zmat
		if(edge == T) {
			fhat <- fhat/edgexy
		}
		if(scale == F) {
			ans$z <- fhat
			ans$scale <- NULL
		}
		else {
			scal <- mean(fhat, na.rm = T) * area(poly)
			ans$z <- fhat/scal
			ans$scale <- scal
		}
		if(pol&(edge==F)){
			xyvec <- cbind(rep(x, ny), rep(y, rep(nx, ny)))
			lpmat1 <- matrix(inout(xyvec, poly), nrow = nx, ncol = 
				ny, byrow = F)
			ans$z[!lpmat1]_NA
		}
	}
		
	}
	if(grid == F) {
		nx <- length(x)
		if(!(nx == length(y)))
			stop("x and y are different lengths")
		ans <- list(0, 0, 0, 0)
		names(ans) <- c("x", "y", "z", "scale")
		ans$x <- x
		ans$y <- y
		ans$scale <- 1

	if(pol&(h==Inf)){ 
		ans$z_rep(1/area(poly),nx) 
		lpin1 <- inout(cbind(x, y), poly)
		ans$z[!lpin1]_NA
	}
	else{
	
		if(edge == T) {
			edgexy <- kdebig2(h, x, y, poly, kernel, grid, inpoly, 
				rings, npt)
		}
		if(kernel == "normal") {
			if(!is.loaded(symbol.For("kdesr3"))){dyn.load(
			"/home/morrisse/libjek/Binreg/Bazza/kdesr3.o")}
			result <- .Fortran("kdesr3",
				h = as.single(h),
				rx = as.single(rx),
				ry = as.single(ry),
				nr <- as.integer(nr),
				x = as.single(x),
				y = as.single(y),
				nx = as.integer(nx))
		}
		if(kernel == "quartic") {
			if(!is.loaded(symbol.For("kdesr6"))){dyn.load(
			"/home/morrisse/libjek/Binreg/Bazza/kdesr6.o")}
			result <- .Fortran("kdesr6",
				h = as.single(h),
				rx = as.single(rx),
				ry = as.single(ry),
				nr <- as.integer(nr),
				x = as.single(x),
				y = as.single(y),
				nx = as.integer(nx))
		}
		fhat <- result$x
		if(edge == T) {
			fhat <- fhat/edgexy
		}
		ans$z <- fhat
		if(pol&(edge==F)){
			lpin1 <- inout(cbind(x, y), poly)
			ans$z[!lpin1]_NA
		}
	}
	}
	ans
}
